package com.jdblabs.timestamper.gui

import com.jdblabs.util.SmartConfig
import com.jdblabs.timestamper.core.TimelineMarker
import com.jdblabs.timestamper.core.TimelineProperties

class TimeStamperMainController {
    // these will be injected by Griffon
    def model
    def view

    def thisMVC
    def syncTimers = [:]

    void mvcGroupInit(Map args) {

        def configFile
        
        logger.trace("Initializing TimeStamperMain MVC...")

        thisMVC = ['model': model, 'view': view, 'controller': this]

        model.notesDialogMVC = buildMVCGroup('NotesDialog', 'notesDialog',
            'mainMVC': thisMVC)

        model.punchcardDialogMVC = buildMVCGroup('PunchcardDialog',
            'punchcardDialog', 'mainMVC': thisMVC)

        // load application properties
        String userHomeDir = System.getProperty('user.home')
        configFile = new File(userHomeDir, ".timestamperrc")

        logger.trace("Reading configuration from {}", configFile.canonicalPath)

        model.config = new SmartConfig(configFile)

        // load the last used timeline file
        String lastUsed = model.config.lastUsed
        if (lastUsed == "") {
            lastUsed = 'timeline.default.properties'
            model.config.setProperty('lastUsed', lastUsed)
        }

        // load the plugin directory
        File pluginDir = model.config.getProperty("pluginDir", new File("plugins"))
        if (!pluginDir.exists()) pluginDir.mkdirs()

        logger.trace("Adding plugin classpath: '{}'", pluginDir.canonicalPath)

        def pluginLoader = new GroovyClassLoader(this.class.classLoader)
        pluginLoader.addURL(pluginDir.toURI().toURL())
        pluginDir.eachFileMatch(/.*\.jar/) { jarfile ->
            pluginLoader.addURL(jarfile.toURI().toURL()) }

        // instantiate plugins
        model.config."plugin.classes".split(',').each { className -> try {
            if (className.trim() == "") return
            def pluginClass = pluginLoader.loadClass(className)
            model.plugins <<pluginClass.newInstance()

            logger.trace("Loaded plugin: {}", className)
        } catch (ClassNotFoundException cnfe) {
            logger.warn("Unable to load plugin ${className}: " +
                "class not found.", cnfe)
        } catch (Throwable t) {
            logger.warn("Unable to load plugin ${className}.", t)
        } }

        // run plugin startup hookd
        model.plugins.each { plugin ->
            wrapPluginCall { plugin.onStartup(thisMVC) } }
        
        logger.trace("Reading Timeline properties from '{}'", lastUsed)

        load(new File(lastUsed))
    }

    def wrapPluginCall(Closure c) {
        try { c() } catch (Throwable t) {
           logger.warn("Plugin threw an exception in ${functionName} " +
                "when passed ${param}:", t) 
            return false
        }
    }

    def load = { File propertiesFile ->
        // pass through plugins
        propertiesFile = model.plugins.inject(propertiesFile) { file, plugin ->
            // call plugin
            def ret = wrapPluginCall { plugin.onTimelineLoad(thisMVC, file) } 
            // if the plugin call succeeded, pass the result, else pass
            // the last one
            ret ? ret : file
        }

        try {
            model.config.lastUsed = propertiesFile.canonicalPath
        } catch (IOException ioe) { logger.error(ioe) }

        // load the properties file
        model.timelineProperties = new TimelineProperties(propertiesFile)

        // load the main timeline
        model.timeline = model.timelineProperties.timeline

        // load the last marker
        model.currentMarker = model.timeline.getLastMarker(new Date())
    }

    def saveas = {
        
    }

    def exitGracefully = { evt = null ->
    
        // hide the frame immediately
        view.frame.visible = false

        logger.trace("Exiting gracefully.")

        // save config
        logger.debug("Config: {}", model.config)
        model.config.save()

        // save timeline and properties
        model.timelineProperties.save()

        // call plugin exit hooks
        model.plugins.each { plugin ->
            wrapPluginCall { plugin.onExit(thisMVC) } }

        logger.trace("Completed graceful shutdown.")

        app.shutdown()
    }

    def newTask = { mark, notes = "No comments.", timestamp = new Date() ->
        TimelineMarker tm = new TimelineMarker(timestamp, mark, notes)

        // pass through the plugins
        tm = model.plugins.inject(tm) { marker, plugin  ->
            def ret = wrapPluginCall { plugin.onNewTask(thisMVC, marker) }
            ret ? ret : marker
        }

        model.timeline.addMarker(tm)
        model.currentMarker = model.timeline.getLastMarker(new Date())

        // auto-persist if enabled
        if (model.timelineProperties.persistOnUpdate)
            model.timelineProperties.save()
    }

    def deleteTask = { marker ->
        // pass through the plugins
        model.plugins.each { plugin  ->
            wrapPluginCall { plugin.onDeleteTask(thisMVC, marker) } }

        model.timeline.removeMarker(marker)
        model.currentMarker = model.timeline.getLastMarker(new Date())

        // auto-persist if enabled
        if (model.timelineProperties.persistOnUpdate)
            model.timelineProperties.save()
    }

}
